@@ -8,6 +8,7 @@
 #include <sys/stat.h>
 #ifndef MSDOS
 #include <sys/wait.h>
+#include <sys/param.h>
 #include <unistd.h>
 #endif
 #include <signal.h>
@@ -24,17 +25,22 @@
 
 #ifdef __STDC__
 #define P(x)	x
+#define HASHIT(x) #x
+#define QUOT(x) HASHIT(x)
 #else
 #define P(x)	()
+/* Well you find something that works! */
+#define QUOT(x) "x"
 #endif
 
 #ifdef MSDOS
-#define LOCALPREFIX	"/linux86"
+#define LOCALPREFIX	/linux86
 #define EXESUF          ".exe"
 #define	R_OK	4		/* Test for read permission.  */
 #define	W_OK	2		/* Test for write permission.  */
 #define	X_OK	1		/* Test for execute permission.  */
 #define	F_OK	0		/* Test for existence.  */
+#define L_TREE	1		/* Use different tree style */
 #else
 #define EXESUF
 #endif
@@ -52,7 +58,7 @@
 #define LD			"ld86" EXESUF
 #define UNPROTO 		"unproto" EXESUF
 
-#ifdef MSDOS
+#ifdef L_TREE
 #define STANDARD_CRT0_0_PREFIX	"~/lib/"
 #define STANDARD_CRT0_3_PREFIX	"~/lib/i386/"
 #define STANDARD_EXEC_PREFIX	"~/lib/"
@@ -137,7 +143,7 @@ PRIVATE struct arg_s tmpargs;	/* = empty */
 PRIVATE char *tmpdir;
 PRIVATE unsigned verbosity;	/* = 0 */
 
-PRIVATE char * localprefix = LOCALPREFIX;
+PRIVATE char * localprefix = QUOT(LOCALPREFIX);
 
 #ifdef REDECLARE_STDC_FUNCTIONS
 void exit P((int status));
@@ -173,7 +179,7 @@ FORWARD char *my_mktemp P((void));
 FORWARD void my_unlink P((char *name));
 FORWARD void outofmemory P((char *where));
 FORWARD int run P((char *in_name, char *out_name, struct arg_s *argp));
-#ifdef MSDOS
+#ifdef L_TREE
 FORWARD void reset_localprefix P((void));
 #endif
 FORWARD void set_trap P((void));
@@ -254,7 +260,7 @@ char **argv;
     				/* GCC also uses 386 how to add -3 too ? */
 #endif
 
-#ifdef MSDOS
+#ifdef L_TREE
     reset_localprefix();
 #endif
     /* Pass 1 over argv to gather compile options. */
@@ -393,6 +399,9 @@ char **argv;
 	        switch(arg[2])
 		{
 		case 'd': /* DOS compile */
+#ifdef BCC86
+		   bits32 = FALSE;
+#endif
 #ifndef CCC
 		   addarg(&ccargs, "-D__MSDOS__");
 #endif
@@ -826,6 +835,7 @@ static struct  aout_exec {
    close(fd);
 }
 
+#ifdef L_TREE
 #ifdef MSDOS
 PRIVATE void reset_localprefix()
 {
@@ -846,6 +856,64 @@ PRIVATE void reset_localprefix()
    else
       free(temp);
 }
+#else
+
+PRIVATE void reset_localprefix()
+{
+   char *ptr, *temp;
+
+   if( *progname == '/' )
+      temp = stralloc(progname);
+   else
+   {
+      char * s, * d;
+      ptr = getenv("PATH");
+      if( ptr==0 || *ptr == 0 ) return;
+      ptr = stralloc(ptr);
+      temp = stralloc("");
+
+      for(d=s=ptr; d && *s; s=d)
+      {
+         char buf[MAXPATHLEN];
+
+	 free(temp);
+         d=strchr(s, ':');
+	 if( d ) *d='\0';
+	 temp = my_malloc(strlen(progname)+strlen(s)+2, "prefixing");
+	 strcpy(temp, s);
+	 strcat(temp, "/");
+	 strcat(temp, progname);
+         if( realpath(temp, buf) != 0 )
+	 {
+	    free(temp);
+	    temp = stralloc(buf);
+         }
+	 if( access(temp, X_OK) == 0 ) break;
+	 d++;
+      }
+      if( s == 0 )
+      {
+         free(temp);
+	 temp = stralloc(progname);
+      }
+      free(ptr);
+   }
+
+   if( (ptr = strrchr(temp, '/')) != 0
+         && temp<ptr-4 && strncmp(ptr-4, "/bin", 4) == 0 )
+   {
+      ptr[-4] = 0;
+      localprefix = temp;
+      if (verbosity > 2)
+      {
+         show_who("localprefix is now ");
+	 writesn(localprefix);
+      }
+   }
+   else
+      free(temp);
+}
+#endif
 #endif
 
 PRIVATE char * expand_tilde(str, canfree)
@@ -966,13 +1034,14 @@ PRIVATE char *my_mktemp()
     static unsigned tmpnum;
 
 #ifdef MSDOS
-    p = template = stralloc2(tmpdir, "/$$YYYXXX");
+    digits = 42;
+    p = template = stralloc2(tmpdir, "/$$YYYYXX");
 #else
+    digits = getpid();
     p = template = stralloc2(tmpdir, "/bccYYYYXXXX");
 #endif
     p += strlen(p);
 
-    digits = getpid();
     while (*--p == 'X')
     {
 	if ((digit = digits % 16) > 9)
